// HyperLink.cpp : implementation file
//

#include "stdafx.h"
#include "HyperLink.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define TIMER_MOUSE 20

CHyperLink::~CHyperLink()
{
	if (m_fontUnderline.m_hObject) this->m_fontUnderline.DeleteObject();
}


BEGIN_MESSAGE_MAP(CHyperLink, CStatic)
	//{{AFX_MSG_MAP(CHyperLink)
	ON_WM_SETCURSOR()
	ON_CONTROL_REFLECT(BN_CLICKED, OnClicked)
	ON_WM_TIMER()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CHyperLink message handlers

BOOL CHyperLink::OnSetCursor(CWnd* pWnd, UINT nHitTest, UINT message) 
{
	HCURSOR h=GetCursor();
	ASSERT(h);
	::SetCursor(h);
	if (!m_bTimer) {
		SetTimer(TIMER_MOUSE,100,NULL);
		
	}
	return TRUE;

	//return CStatic::OnSetCursor(pWnd, nHitTest, message);
}

void CHyperLink::OnClicked() 
{
	CString strURL=m_strURL;

	if (strURL.IsEmpty()) {
		GetWindowText(strURL);
	}
	
	CString s;
	if (strURL.Find("@",0)>0) {//Email Address
		s="mailto:"+strURL;
	}
	else {//URL Address
		s=strURL;
	}
	
	const DWORD dwErr=::GetLastError();
	SetLastError(0);
	const HINSTANCE h=::ShellExecute(this->m_hWnd,"open",s,NULL,NULL,SW_SHOW);	
	
	if ((int)h<=31) {
		CString s;
		s.Format("Cannot open [%s]\r\n%s",strURL,GetLastError());
		AfxMessageBox(s,MB_ICONEXCLAMATION);
	}

	SetLastError(dwErr);
}

CString CHyperLink::GetLastError() const
{
	CString s;
	FormatMessage( 
    FORMAT_MESSAGE_FROM_SYSTEM | 
    FORMAT_MESSAGE_IGNORE_INSERTS,
    NULL,
    ::GetLastError(),
    MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
    s.GetBufferSetLength(1025),
    1024,
    NULL 
);
	s.ReleaseBuffer();
	return s;
}

void CHyperLink::OnTimer(UINT nID)
{
	if (TIMER_MOUSE) {
		if ((IsUnderCursor()) && (!m_bUnderline)) {
			SetFont(&GetUnderlineFont(),TRUE);
			m_bUnderline=true;
		}
		else if ((!IsUnderCursor()) && (m_bUnderline)) {
			SendMessage(WM_SETFONT,(WPARAM)GetStockObject(DEFAULT_GUI_FONT),TRUE);
			m_bUnderline=false;
		}

		return;
	}
	CStatic::OnTimer(nID);
}

CFont& CHyperLink::GetUnderlineFont()
{
	if (!m_fontUnderline.m_hObject) 
		VERIFY(CreateFont(m_fontUnderline));
	return m_fontUnderline;

}

bool CHyperLink::CreateFont(CFont &font)
{
	ASSERT(NULL==font.m_hObject);
	HFONT h=(HFONT)SendMessage(WM_GETFONT);
	ASSERT(h);
	if (!h) {
		h=(HFONT)GetStockObject(SYSTEM_FONT);
		ASSERT(h);
	}
	if (!h) return false;
	LOGFONT lf;
	GetObject((HGDIOBJ)h,sizeof(LOGFONT),(void*)&lf);
	lf.lfUnderline=TRUE;
	lf.lfWeight=FW_NORMAL;

	if (font.CreateFontIndirect(&lf))
		return true;
	else
		return false;
}

bool CHyperLink::IsUnderCursor()
{
	POINT pt;
	GetCursorPos(&pt);
	
	CRect r;
	GetWindowRect(r);

	return TRUE==r.PtInRect(pt);
}

CHyperLink::CHyperLink(LPCTSTR lpszURL)
{
	m_strURL=CString(lpszURL);
	Init();
}

CHyperLink::Init()
{
	m_fontUnderline.m_hObject=NULL;
	m_bUnderline=m_bTimer=false;	
}

HCURSOR CHyperLink::GetCursor()
{
 struct {
	enum { cxWidth = 32, cyHeight = 32 };
	int xHotSpot;
	int yHotSpot;
	unsigned char arrANDPlane[cxWidth * cyHeight / 8];
	unsigned char arrXORPlane[cxWidth * cyHeight / 8];
 } Data = {
	5, 0, 
	{
		0xF9, 0xFF, 0xFF, 0xFF, 0xF0, 0xFF, 0xFF, 0xFF, 0xF0, 0xFF, 0xFF, 0xFF, 0xF0, 0xFF, 0xFF, 0xFF, 
		0xF0, 0xFF, 0xFF, 0xFF, 0xF0, 0x3F, 0xFF, 0xFF, 0xF0, 0x07, 0xFF, 0xFF, 0xF0, 0x01, 0xFF, 0xFF, 
		0xF0, 0x00, 0xFF, 0xFF, 0x10, 0x00, 0x7F, 0xFF, 0x00, 0x00, 0x7F, 0xFF, 0x00, 0x00, 0x7F, 0xFF, 
		0x80, 0x00, 0x7F, 0xFF, 0xC0, 0x00, 0x7F, 0xFF, 0xC0, 0x00, 0x7F, 0xFF, 0xE0, 0x00, 0x7F, 0xFF, 
		0xE0, 0x00, 0xFF, 0xFF, 0xF0, 0x00, 0xFF, 0xFF, 0xF0, 0x00, 0xFF, 0xFF, 0xF8, 0x01, 0xFF, 0xFF, 
		0xF8, 0x01, 0xFF, 0xFF, 0xF8, 0x01, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 
		0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 
		0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF
	},
	{
		0x00, 0x00, 0x00, 0x00, 0x06, 0x00, 0x00, 0x00, 0x06, 0x00, 0x00, 0x00, 0x06, 0x00, 0x00, 0x00, 
		0x06, 0x00, 0x00, 0x00, 0x06, 0x00, 0x00, 0x00, 0x06, 0xC0, 0x00, 0x00, 0x06, 0xD8, 0x00, 0x00, 
		0x06, 0xDA, 0x00, 0x00, 0x06, 0xDB, 0x00, 0x00, 0x67, 0xFB, 0x00, 0x00, 0x77, 0xFF, 0x00, 0x00, 
		0x37, 0xFF, 0x00, 0x00, 0x17, 0xFF, 0x00, 0x00, 0x1F, 0xFF, 0x00, 0x00, 0x0F, 0xFF, 0x00, 0x00, 
		0x0F, 0xFE, 0x00, 0x00, 0x07, 0xFE, 0x00, 0x00, 0x07, 0xFE, 0x00, 0x00, 0x03, 0xFC, 0x00, 0x00, 
		0x03, 0xFC, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
	}
};
	static HCURSOR hCur=NULL;
	if (!hCur) hCur=::CreateCursor(AfxGetInstanceHandle(),Data.xHotSpot
		,Data.yHotSpot
		,Data.cxWidth 
		,Data.cyHeight 
		,Data.arrANDPlane
		,Data.arrXORPlane);
	return hCur;

}
