// RichUtilApi.cpp: implementation of the RichUtilApi class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "RichUtilBase.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#define GROWBY 1024*100

namespace {
	class CBufferFile:public CMemFile {
	public:
		CBufferFile() : CMemFile(GROWBY) {}
	};

	struct CALLBACKPARAM {
		RichUtilBase::PFUNC_CALLBACK pfnCallBack;
		long lCur;
		long lMax;
		LPARAM lParam;
		CBufferFile* pFile;
		CALLBACKPARAM(): pfnCallBack(NULL),lMax(0),lParam(0),pFile(NULL),lCur(0) {}
	};		
};//namespace

static
DWORD CALLBACK In(DWORD dwCookie, LPBYTE pbBuff, LONG cb, LONG FAR *pcb)
{
	CALLBACKPARAM* pParam=reinterpret_cast<CALLBACKPARAM*>(dwCookie);
	//CBufferFile* f=(CBufferFile*)dwCookie;
	__try {
		*pcb=pParam->pFile->Read(pbBuff,cb);
		if (pParam->pfnCallBack) {
			ASSERT(!IsBadCodePtr((FARPROC)pParam->pfnCallBack));
			pParam->lCur+=(*pcb);
			pParam->pfnCallBack(pParam->lCur,pParam->lMax,pParam->lParam);
		}
	}
	__except(EXCEPTION_EXECUTE_HANDLER) {
		ASSERT(FALSE);
		TRACE("WARNING __except(EXCEPTION_EXECUTE_HANDLER) in In Callback (RichUtilBase.cpp)\n");
	}
	return 0;
}

#define RICH_MUL 4//   ???

bool RichUtilBase::TextIn(const HWND hwndRich
					   , const bool bSelection
					   , const bool bAsPlainText
					   , LPCTSTR lpszText
					   ,PFUNC_CALLBACK pfnCallBack
					   ,LPARAM lParam)
{
	ASSERT(IsWindow(hwndRich));
	ASSERT(!IsBadStringPtr(lpszText,INFINITE));
	if (!IsWindow(hwndRich)) return false;
	if (IsBadStringPtr(lpszText,INFINITE)) return false;

	const DWORD dwStringLen=bAsPlainText ? strlen(lpszText) : strlen(lpszText)*RICH_MUL;
	if (dwStringLen>32767)
		SendMessage(hwndRich,EM_EXLIMITTEXT,0,dwStringLen);

	CBufferFile f;	
	//   
	f.Attach((unsigned char*)lpszText,strlen(lpszText));

	CALLBACKPARAM param;
	param.pFile=&f;
	param.pfnCallBack=pfnCallBack;
#ifdef _DEBUG
	if (param.pfnCallBack)
		ASSERT(!IsBadCodePtr((FARPROC)param.pfnCallBack));
#endif
	param.lParam=lParam;
	//   CALLBACK
	if (pfnCallBack) {
		//  .  
		param.lMax=strlen(lpszText);
	}

	EDITSTREAM es;
	es.dwCookie=(DWORD)&param;
	es.dwError=0;
	
	es.pfnCallback=In;

	try {
		WPARAM wParam=bAsPlainText ? SF_TEXT:SF_RTF;
		if (bSelection) wParam|=SFF_SELECTION;
		
		//rich.StreamIn(wParam,es);
		SendMessage(hwndRich,EM_STREAMIN,wParam,(LPARAM)&es);
	}

	catch (...) {
		ASSERT(FALSE);
		es.dwError=1;
		f.Detach();
		return FALSE;
	}
	f.Detach();
	ASSERT(0==es.dwError);
	return (0==es.dwError);
}

static
DWORD CALLBACK Out(DWORD dwCookie, LPBYTE pbBuff, LONG cb, LONG FAR *pcb)
{
	ASSERT(!IsBadReadPtr((void*)pbBuff,cb));
	CALLBACKPARAM* const pParam=reinterpret_cast<CALLBACKPARAM*>(dwCookie);
	ASSERT(!IsBadReadPtr((void*)pParam,sizeof(CALLBACKPARAM)));
	pParam->pFile->Write(pbBuff,cb);
	*pcb=cb;
	if (pParam->pfnCallBack) {
		ASSERT(!IsBadCodePtr((FARPROC)pParam->pfnCallBack));
		pParam->lCur+=cb;
		if (pParam->lCur>pParam->lMax) pParam->lMax=pParam->lCur+1;
		pParam->pfnCallBack(pParam->lCur,pParam->lMax,pParam->lParam);
	}
	return 0;
}

CString RichUtilBase::TextOut(const HWND hwndRich
						   , const bool bSelection
						   ,const bool bAsPlainText
						   , const bool /*bAsUnicode*/
						   ,PFUNC_CALLBACK pfnCallBack
						   ,LPARAM lParam)
{
	ASSERT(IsWindow(hwndRich));
	if (!IsWindow(hwndRich))
		return TEXT("");

	CBufferFile f;
	CALLBACKPARAM param;
	param.pFile=&f;
	//     
	if (!bSelection) {
		param.pfnCallBack=pfnCallBack;
		param.lParam=lParam;
		if (param.pfnCallBack) {
			ASSERT(!IsBadCodePtr((FARPROC)param.pfnCallBack));
			//  . 
			GETTEXTLENGTHEX len;
			len.codepage=CP_ACP;
			len.flags=GTL_USECRLF|GTL_PRECISE;
			if (!bAsPlainText) 
				len.flags|=GTL_NUMBYTES;
			else
				len.flags|=GTL_NUMCHARS;
			//param.lMax=rich.SendMessage(EM_GETTEXTLENGTHEX,(WPARAM)&len)*2;
			param.lMax=SendMessage(hwndRich,EM_GETTEXTLENGTHEX,(WPARAM)&len,0)*2;
			//SendMessage(hwndRich,EM_EXLIMITTEXT,0,(ULONG)abs(param.lMax+1));
		}
	}

/*	long lLen;
	if (bSelection) {
		CHARRANGE crSel={0};
		SendMessage(hwndRich,EM_EXGETSEL,0,(LPARAM)&crSel);
		lLen=abs(crSel.cpMax-crSel.cpMin);
	}
	else
		lLen=param.lMax;

	if (!bAsPlainText) lLen*=RICH_MUL;
	if (lLen>32767)
		SendMessage(hwndRich,EM_EXLIMITTEXT,0,lLen);
		*/
	
	EDITSTREAM es;
	es.dwCookie=(DWORD)&param;
	es.dwError=0;
	es.pfnCallback=Out;

	WPARAM wParam=(bAsPlainText ? SF_TEXT : SF_RTF);

	if (bSelection) wParam|=SFF_SELECTION;
	//  ,      2000  9
	/*
	if (bAsUnicode) 
		wParam|=SFF_UNICODE;
	else
		wParam|=(CP_ACP<<16|SF_USECODEPAGE);
	*/

	SendMessage(hwndRich, EM_STREAMOUT,wParam,(LPARAM)&es);
	ASSERT(0==es.dwError);
	const DWORD n=f.GetLength();
	
	CString s;
	memcpy(s.GetBufferSetLength(n+1),f.Detach(),n);
	s.ReleaseBuffer(n);
	
	const DWORD dwLen=s.GetLength();
	ASSERT(dwLen==n);

	return s;
}


bool RichUtilBase::GetVisibleRange(const HWND hwnd, long &lStart, long &lEnd)
{
	ASSERT(IsWindow(hwnd));
	if (!IsWindow(hwnd)) return false;

	const long lFirstLine=SendMessage(hwnd,EM_GETFIRSTVISIBLELINE,0,0);

	RECT r;
	VERIFY(GetClientRect(hwnd,&r));
	
	POINTL ptl;
	ptl.x=r.right;
	ptl.y=r.bottom;
	
	lEnd=SendMessage(hwnd,EM_CHARFROMPOS,0,(LPARAM)&ptl);
	lStart=SendMessage(hwnd,EM_LINEINDEX,lFirstLine,0);
	
	return true;
}

bool RichUtilBase::GetVisibleRange(const HWND hwnd, CHARRANGE &cr)
{
	return GetVisibleRange(hwnd,cr.cpMin,cr.cpMax);
}

static
bool IsRichEdit10(const HWND hwnd)
{
	ASSERT(IsWindow(hwnd));

	const HMODULE hUser=GetModuleHandle(TEXT("user32.dll"));
	ASSERT(hUser);

	typedef UINT (WINAPI *RealGetWindowClassProc)(HWND, LPTSTR,UINT);	
	RealGetWindowClassProc pfn=(RealGetWindowClassProc)
		GetProcAddress(hUser,TEXT("RealGetWindowClass"));
	if (!pfn) {
		return true;
	}
	else {
		CString s;
		pfn(hwnd,s.GetBufferSetLength(MAX_PATH+1),MAX_PATH);
		s.ReleaseBuffer();
		
		return (0==s.CompareNoCase(TEXT("RichEdit")));		
	}
}

bool RichUtilBase::SetSelToEnd(const HWND hwndRich)
{
	ASSERT(IsWindow(hwndRich));
	if (!IsWindow(hwndRich))
		return false;

	
	CHARRANGE cr;
	if (IsRichEdit10(hwndRich)) {
		const long lLen=SendMessage(hwndRich,WM_GETTEXTLENGTH,0,0);
		cr.cpMin=cr.cpMax=lLen;
	}
	else
		cr.cpMin=cr.cpMax=-1;

	SendMessage(hwndRich,EM_EXSETSEL,0,(LPARAM)&cr);

	return true;
}
